/*
 * Copyright 2017 Dmitry Ivanov
 *
 * This file is part of libquentier
 *
 * libquentier is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, version 3 of the License.
 *
 * libquentier is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with libquentier. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LIB_QUENTIER_UTILITY_VERSION_INFO_H
#define LIB_QUENTIER_UTILITY_VERSION_INFO_H

#include <quentier/utility/Linkage.h>
#include <QString>

/**
 * This file contains automatically generated code including macro definitions
 * and functions which determine configuration-specific properties of libquentier
 */

/**
 * This macro is set to true if libquentier uses QtWebEngine component;
 * it requires specialized deployment, so this definition can tell whether
 * it is required for libquentier
 */
#define LIB_QUENTIER_USE_QT_WEB_ENGINE 1

/**
 * This macro is set to true if libquentier was built with AuthenticationManager class
 * for OAuth handling
 */
#define LIB_QUENTIER_HAS_AUTHENTICATION_MANAGER 1

/**
 * This macro is set to true if libquentier was built with NoteEditor and INoteEditorBackend classes
 * (as well as their implementations)
 */
#define LIB_QUENTIER_HAS_NOTE_EDITOR 1

/**
 * This macro is set to a string holding the information about the current libquentier build
 */
#define LIB_QUENTIER_BUILD_INFO "unknown"

/**
 * This macro is set to the major version number of libquentier
 */
#define LIB_QUENTIER_VERSION_MAJOR 0

/**
 * This macro is set to the minor version of libquentier
 */
#define LIB_QUENTIER_VERSION_MINOR 4

/**
 * This macro is set to the patch version of libquentier
 */
#define LIB_QUENTIER_VERSION_PATCH 0

namespace quentier {

/**
 * @return          Major version of libquentier
 */
int QUENTIER_EXPORT libquentierVersionMajor();

/**
 * @return          Minor version of libquentier
 */
int QUENTIER_EXPORT libquentierVersionMinor();

/**
 * @return          Patch version of libquentier
 */
int QUENTIER_EXPORT libquentierVersionPatch();

/**
 * @return          String containing information about the current libquentier build
 */
QString QUENTIER_EXPORT libquentierBuildInfo();

/**
 * @return          String representing the version of Qt against which
 *                  libquentier was built
 */
QString QUENTIER_EXPORT libquentierBuiltWithQtVersion();

/**
 * @return          True if the current build if libquentier has note editor
 *                  included, false otherwise
 */
bool QUENTIER_EXPORT libquentierHasNoteEditor();

/**
 * @return          True if the current build of libquentier has authentication
 *                  manager included, false otherwise
 */
bool QUENTIER_EXPORT libquentierHasAuthenticationManager();

/**
 * @return          True if the current build of libquentier uses QtWebEngine
 *                  component for note editor and/or authentication manager,
 *                  false otherwise (if neither note editor nor authentication
 *                  manager are present or if QtWebKit is used for these instead
 *                  of QtWebEngine)
 */
bool QUENTIER_EXPORT libquentierUsesQtWebEngine();

} // namespace quentier

#endif // LIB_QUENTIER_UTILITY_VERSION_INFO_H
